import asyncio

from aiostreammagic.stream_magic import StreamMagicClient
from aiostreammagic.models import CallbackType

HOST = "192.168.x.x"  # Replace with your StreamMagic device's IP address


async def on_state_change(
    client: StreamMagicClient, callback_type: CallbackType
) -> None:
    """Called when new information is received."""
    print(f"Callback Type: {callback_type} {client.is_connected()}")
    print(f"System info: {client.info}")
    print(f"Sources: {client.sources}")
    print(f"State: {client.state}")
    print(f"Play State: {client.play_state}")
    print(f"Now Playing: {client.now_playing}")
    print(f"Display: {client.display}")
    print(f"Preset List: {client.preset_list}")


async def main() -> None:
    """Subscribe demo entrypoint."""
    client = StreamMagicClient(HOST)
    await client.register_state_update_callbacks(on_state_change)
    await client.connect()

    # Play media using the unit's front controls or StreamMagic app
    await asyncio.sleep(60)

    await client.disconnect()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())
    loop.close()



"""
.. include:: ../README.md
"""

from .exceptions import StreamMagicError, StreamMagicConnectionError
from .models import (
    Info,
    PlayStateMetadata,
    PlayState,
    State,
    Source,
    NowPlaying,
    ControlBusMode,
    TransportControl,
    RepeatMode,
    ShuffleMode,
    EQBand,
    UserEQ,
    EQFilterType,
    Audio,
    EQ_PRESETS,
)
from .stream_magic import StreamMagicClient

__all__ = [
    "StreamMagicClient",
    "StreamMagicError",
    "StreamMagicConnectionError",
    "Info",
    "Source",
    "State",
    "PlayState",
    "PlayStateMetadata",
    "NowPlaying",
    "ControlBusMode",
    "TransportControl",
    "ShuffleMode",
    "RepeatMode",
    "EQBand",
    "UserEQ",
    "EQFilterType",
    "Audio",
    "EQ_PRESETS",
]



"""Example usage of EQ presets."""

import asyncio

from aiostreammagic import (
    StreamMagicClient,
    EQBand,
    EQ_PRESETS,
)
from aiostreammagic.exceptions import StreamMagicError


async def main() -> None:
    """Example of using EQ presets."""
    # List all available presets
    print("Available EQ presets:")
    for preset_name in EQ_PRESETS.keys():
        print(f"  - {preset_name}")

    # Get preset gain values
    bass_boost_gains = EQ_PRESETS["bass_boost"]
    print(f"\nBass boost gains: {bass_boost_gains}")

    # Set EQ using presets or custom values
    async with StreamMagicClient("192.168.x.x") as client:
        try:
            # Use set_equalizer_preset
            await client.set_equalizer_preset("bass_boost")
            print("\nApplied bass_boost preset")

            # Build custom EQ from gains
            custom_gains = [1.0, 0.5, 0.0, 0.0, 0.0, 0.5, 1.0]
            bands = [EQBand(index=i, gain=gain) for i, gain in enumerate(custom_gains)]
            await client.set_equalizer_params(bands)
            print("Applied custom EQ gains")

        except StreamMagicError as e:
            print(f"\nError: {e}")


if __name__ == "__main__":
    asyncio.run(main())



"""Utility functions for StreamMagic."""

from typing import Optional

from aiostreammagic.models import EQBand


def eq_bands_to_param_string(bands: list[EQBand]) -> str:
    """Format EQ bands as required by the API.

    Args:
        bands: List of EQ bands to format

    Returns:
        Pipe-separated string of band parameters in format:
        "index,filter,freq,gain,q|index,filter,freq,gain,q|..."
    """

    def fmt(val: object, float_fmt: Optional[str] = None) -> str:
        if val is None:
            return ""
        if float_fmt and isinstance(val, float):
            return float_fmt.format(val)
        return str(val)

    return "|".join(
        f"{fmt(band.index)},{fmt(band.filter)},{fmt(band.freq)},{fmt(band.gain, '{:.1f}')},{fmt(band.q, '{:.2f}')}"
        for band in bands
    )
